Общее описание внутреннего устройства платы L502. {#sect_gen_descr}
==============================================================

Данный раздел предоставляет дополнительные сведения о низкоуровневом устройстве платы L502, которые необходимы для создания своих драйверов или своей прошивки сигнального процессора. В этом разделе в первую очередь будет рассматриваться низкоуровневое устройство с точки зрения программиста, а не непосредственно аппаратной реализации.

Ниже представлена общая схема L502 с точки зрения взаимодействия различных составных частей модуля L502.

\image latex l502_ll_sch.jpeg "Взаимодействие функциональных блоков модуля L502" width=\textwidth
\image html l502_ll_sch.jpeg "Взаимодействие функциональных блоков модуля L502"



Как видно из рисунка, модуль состоит из следующих блоков:
1. ПЛИС. В нем реализовано:
    - Вся логика интерфейса PCI-Express
    - Логика доступа к памяти сигнального процессора с ПК через регистры PCI
    - Блок DMA для прямой передачи данных АЦП/DIN в память ПК и чтения данных ЦАП/DOUT из памяти ПК.
    - Блок управления вводом-выводом. В нем реализовано две группы регистров:
        - \f$ $\hyperref[tab:regIoHard]{IO\_HARD} $ \f$  - для настройки параметров сбора данных (логическая таблица, делитель частоты, режим синхронизации и т.д.)
        - \f$ $\hyperref[tab:regIoArith]{IO\_ARITH} $ \f$ - для дополнительной обработки (аппаратное применение калибровочных коэффициентов)
    - Реализованы интерфейсы к Flash-памяти, Сигнальному процессору и гальваноотвязанной части.
2. Flash-память SST25VF16B --- память на 2 Мбайта. Старшая половина пользователю не доступна --- в ней хранится прошивка ПЛИС, ее резервная копия, калибровочные коэффициенты и серийный номер изделия. Младшая половина доступна пользователю для чтения и записи, и может быть использована для каких-либо своих целей. Следует однако учитывать, что Flash-память доступна только для ПК, и к ней нет прямого доступа у сигнального процессора.
3. Сигнальный процессора [ADSP-BF523](http://www.analog.com/en/processors-dsp/blackfin/adsp-bf523/products/product.html). Может быть использован для написания собственных программ обработки данных внутри модуля для расширения возможностей L502.
4. PLDA - ПЛИС, управляющий гальваноотвязанной частью. Именно на гальваноотвязанной части находятся все сигнальные входы и выходы модуля, АЦП и ЦАП. Ими непосредственно управляет PLDA. Сам PLDA управляется через гальваноразвязанный интерфейс основным ПЛИС и логика работы PLDA скрыта от пользователя, т.к. пользователю доступно управление сбором данных только через регистры блока ввода-вывода основного ПЛИС.

Все управление модулем со стороны ПК идет через регистры PCI-Express (с программной стороны аналогичны регистрам PCI) и описано подробно [в  следующем разделе](@ref sect_pc_ctl).

Как видно из рисунка, плата может работать в двух режимах: штатном (всю обработку выполняет ПЛИС) и DSP-режиме. В зависимости от режима меняются путь, который проходят потоки данных, а также доступ к управляющим регистрам ввода-вывода.

Если в штатном режиме данные, приходящие по гальваноотвязанному интерфейсу (АЦП и DIN), передаются сразу в буфер блока DMA для последующей записи по интерфейсу PCI-Express непосредственно в память ПК, то в DSP-режиме данные передаются через SPORT0 в BlackFin, который может обработать эти данные и, при желании, поставить их на передачу в ПК. В этом случае ПЛИС по HostDMA прочитает данные из памяти BlackFin и передаст их через блок DMA в память ПК.

Аналогично для потока на вывод в режиме DSP данные, прочитанные блоком DMA из памяти ПК, будут записаны через интерфейс HostDMA в память BlackFin, где могут быть обработаны сигнальным процессором и переданы через SPORT0 обратно в ПЛИС для дальнейшего их вывода на ЦАП или цифровые выходы.

Также при переходе в режим DSP изменяется право доступа к регистрам \f$ $\hyperref[tab:regIoHard]{IO\_HARD} $ \f$ и \f$ $\hyperref[tab:regIoArith]{IO\_ARITH} $ \f$. Если в штатном режиме данные регистры доступны для программирования со стороны ПК как обычные регистры PCI, то в режиме DSP управление этими регистрами передается сигнальному процессору, который может через интерфейс SPI записывать и считывать их содержимое. При этом состав регистров и относительные адреса остаются теми же. 


<B> Внимание! </B>: Здесь и далее названия направлений передачи потоков (ввод или вывод) даются относительно ПК. То есть под потоком ввода (IN) понимается поток данных АЦП/DIN, а под потоком вывода (OUT) --- поток данных ЦАП/DOUT, даже когда речь идет о прошивке сигнального процессора.