#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "radix64.h"


static char f_replace_str[] = "@DATA@";
#define RPLASE_STR_SIZE  (sizeof(f_replace_str)-1)

#define READ_SIZE 4096

static int f_file_wr_exact(FILE* f, char* buf, size_t size)
{
    size_t wr_size;
    if (size==0)
        return 0;

    wr_size = fwrite(buf, 1, size, f);
    if (wr_size!=size)
    {
        fprintf(stderr, "File write error..\n");
        return -5;
    }
    return 0;
}

static int f_write_data(FILE* in, FILE* out)
{
    static unsigned char in_buf[3*256];
    static char out_buf[4*256];
    int err = 0;

    do
    {
        size_t rd_size = fread(in_buf, 1, sizeof(in_buf), in);
        if ((rd_size != sizeof(in_buf)) && (ferror(in)))
        {
            fprintf(stderr, "File read error..\n");
            err = -4;
        }
        else if (rd_size)
        {
            size_t size = Radix64_Encode(in_buf, rd_size, out_buf);
            err = f_file_wr_exact(out, out_buf, size);
        }
    } while (!err && !feof(in));
    return err;
}

int main(int argc, char **argv)
{
    static char buf[READ_SIZE+sizeof(f_replace_str)];

    int err = 0;
    FILE *info_file=NULL, *firm_file = NULL, *out_file = NULL;
    if (argc < 4)
    {
        fprintf(stderr, "Insufficient argument. Usage: lxfirm_prepare <firm_file> <info_file> <result_file>\n");
        return -1;
    }


    firm_file = fopen(argv[1], "rb");
    if (firm_file==NULL)
    {
        fprintf(stderr, "Cannot open firmware file (%s)!\n", argv[1]);
        err = -2;
    }

    if (!err)
    {
        info_file = fopen(argv[2], "r");
        if (info_file==NULL)
        {
            fprintf(stderr, "Cannot open info file (%s)!\n", argv[2]);
            err = -2;
        }
    }

    if (!err)
    {
        out_file = fopen(argv[3], "w");
        if (out_file==NULL)
        {
            fprintf(stderr, "Cannot create output file (%s)!\n", argv[3]);
            err = -2;
        }
    }


    if (!err)
    {
        size_t proc_size=0, rd_size;

        do
        {

            rd_size = fread(&buf[proc_size], 1, sizeof(buf)-proc_size, info_file);
            if (ferror(info_file))
            {
                fprintf(stderr, "File read error!!\n");
                err = -3;
            }
            else
            {
                size_t i;
                size_t wr_pos = 0;

                proc_size = rd_size= rd_size+proc_size;;
                if (proc_size > READ_SIZE)
                    proc_size = READ_SIZE;

                for (i=0; i < proc_size; i++)
                {
                    if ((buf[i]==f_replace_str[0]) && ((i+RPLASE_STR_SIZE)<=rd_size))
                    {
                        //int repl_size = RPLASE_STR_SIZE;
                        if (!memcmp(&buf[i], f_replace_str, RPLASE_STR_SIZE))
                        {

                            size_t wr_size= (i-wr_pos);
                            err = f_file_wr_exact(out_file, &buf[wr_pos], wr_size);
                            if (!err)
                            {
                                err = f_write_data(firm_file, out_file);
                            }

                            if (!err)
                            {
                                i+=RPLASE_STR_SIZE-1;
                                wr_pos = i+1;
                            }
                        }
                    }
                }

                if (!err && (wr_pos != proc_size))
                {
                    err = f_file_wr_exact(out_file, &buf[wr_pos], proc_size-wr_pos);
                }

                if (!err)
                {
                    if (proc_size!=rd_size)
                    {
                        memmove(buf, &buf[proc_size], rd_size-proc_size);
                    }
                    proc_size = rd_size - proc_size;
                }
            }
        }
        while (!err && !feof(info_file));

        if (!err)
        {
            err = f_file_wr_exact(out_file, buf, proc_size);
        }
    }

    if (firm_file)
        fclose(firm_file);
    if (info_file)
        fclose(info_file);
    if (out_file)
        fclose(out_file);


    return err;
}
